<?php

namespace FM\Factory;

use Doctrine\ORM\EntityManager;

class SolicitacaoFactory {
    protected $connection;

    public function __construct(EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
    }
    
    // Busca as solicitações
    public function getRequests() {
        $sql = "SELECT SOL.NRSOLICITACAO, SOL.NMSOLICITACAO, SOL.IDSITUACAO, FUNC.NMFUNCIONARIO, TO_CHAR(SOL.DTSOLICITACAO, 'DD/MM/YYY HH24:MI') AS DTSOLICITACAO, LOC.NMLOCALCLIENTE, LOCP.NMLOCALCLIENTE AS NMLOCALCLIENTEPAI
                    FROM FCM_SOLICITACAO SOL
                        JOIN FCM_LOCALCLI LOC ON LOC.NRLOCALCLIENTE = SOL.NRLOCALCLIENTE
                        JOIN FCM_LOCALCLI LOCP ON LOCP.NRLOCALCLIENTE = LOC.NRLOCALCLIENTEPAI
                        LEFT JOIN FCM_FUNCIONARIO FUNC ON FUNC.NRFUNCIONARIO = SOL.NRFUNCRESPONSAVEL"; // Busca tb o local e o pai do local

        $std = $this->connection->prepare($sql);
        $std->execute();
        return $std->fetchAll();
    }
    
    // Busca os tipos de solicitação
    public function getRequestTypes() {
        $sql = "SELECT NRTIPOSOLICITACAO, NMTIPOSOLICITACAO, 
                        (CASE IDPRIORIDADEPADR
                            WHEN '1' THEN 'Baixa'
                            WHEN '2' THEN 'Média'
                            WHEN '3' THEN 'Alta'
                            WHEN '4' THEN 'Urgente'
                            ELSE NULL
                        END) AS IDPRIORIDADEPADR
                    FROM FCM_TIPOSOLICITACAO";

        $std = $this->connection->prepare($sql);
        $std->execute();
        return $std->fetchAll();
    }
    
    // Busca as informações de uma solicitação específica
    public function getRequestInfo($nrsolicitacao) {
        $sql = "SELECT SOL.NRSOLICITACAO, SOL.NMSOLICITACAO, TO_CHAR(SOL.DTSOLICITACAO, 'DD/MM/YYYY HH24:MI') AS DTSOLICITACAO, SOL.NRLOCALCLIENTE, LOC.NMLOCALCLIENTE, SOL.NRFUNCABERTURA, FUNC.NMFUNCIONARIO AS NMFUNCABERTURA, SOL.DSSOLICITACAO,
                        (CASE SOL.IDSITUACAO
                            WHEN '1' THEN 'Pendente'
                            WHEN '2' THEN 'Em andamento'
                            WHEN '3' THEN 'Aguardando Avaliação'
                            WHEN '4' THEN 'Concluída'
                            ELSE NULL
                        END) AS IDSITUACAO,
                        (CASE SOL.IDPRIORIDADE
                            WHEN '1' THEN 'Baixa'
                            WHEN '2' THEN 'Média'
                            WHEN '3' THEN 'Alta'
                            WHEN '4' THEN 'Urgente'
                            ELSE 'A classificar'
                        END) AS IDPRIORIDADE,
                        (CASE  NVL(TSOL.NMTIPOSOLICITACAO, 0)
                            WHEN '0' THEN 'A classificar'
                            ELSE TSOL.NMTIPOSOLICITACAO
                        END) AS NMTIPOSOLICITACAO, TSOL.NRTIPOSOLICITACAO,
                        (CASE  NVL(FUNC2.NMFUNCIONARIO, 0)
                            WHEN '0' THEN 'A classificar'
                            ELSE FUNC2.NMFUNCIONARIO
                        END) AS NMFUNCRESPONSAVEL, SOL.NRFUNCRESPONSAVEL
                    FROM FCM_SOLICITACAO SOL
                    JOIN FCM_FUNCIONARIO FUNC ON FUNC.NRFUNCIONARIO = SOL.NRFUNCABERTURA
                    LEFT JOIN FCM_FUNCIONARIO FUNC2 ON FUNC2.NRFUNCIONARIO = SOL.NRFUNCRESPONSAVEL
                    JOIN FCM_LOCALCLI LOC ON LOC.NRLOCALCLIENTE = SOL.NRLOCALCLIENTE
                    LEFT JOIN FCM_TIPOSOLICITACAO TSOL ON TSOL.NRTIPOSOLICITACAO = SOL.NRTIPOSOLICITACAO
                 WHERE SOL.NRSOLICITACAO = {$nrsolicitacao}";

        $std = $this->connection->prepare($sql);
        $std->execute();
        return $std->fetchAll();
    }
    
    // Busca o histórico da solicitação
    public function getRequestHistoric($nrsolicitacao) {
        $sql = "SELECT HIST.NRFUNCSOLICITANTE, HIST.NMSOLICITACAO, FUNC.NMFUNCIONARIO, TO_CHAR(HIST.DTHISTSOLICITACAO, 'DD/MM/YYYY HH24:MI') AS DTHISTSOLICITACAO, LOC.NRLOCALCLIENTE, LOC.NMLOCALCLIENTE, HIST.DSSOLICITACAO,  HIST.DTINCLUSAO,
                        (CASE HIST.IDTIPOHISTORICO
                                WHEN '1' THEN 'Criação de Solicitação'
                                WHEN '2' THEN 'Solicitação Classificada'
                                WHEN '3' THEN 'Solicitação em execução'
                                WHEN '4' THEN 'Solicitação Avaliada'
                                ELSE NULL
                        END) AS IDTIPOHISTORICO          
                    FROM FCM_SOLICITACAOHISTORICO HIST
                    JOIN FCM_FUNCIONARIO FUNC ON FUNC.NRFUNCIONARIO = HIST.NRFUNCSOLICITANTE
                    JOIN FCM_SOLICITACAO SOL ON SOL.NRSOLICITACAO = HIST.NRSOLICITACAO
                    JOIN FCM_LOCALCLI LOC ON LOC.NRLOCALCLIENTE = SOL.NRLOCALCLIENTE
                 WHERE HIST.NRSOLICITACAO = {$nrsolicitacao}
                ORDER BY HIST.DTINCLUSAO DESC";

        $std = $this->connection->prepare($sql);
        $std->execute();
        return $std->fetchAll();
    }

    // Insere um novo tipo de requisição
    public function insertRequestType($typeParams) {
        $sql = "INSERT INTO FCM_TIPOSOLICITACAO (NRTIPOSOLICITACAO, NMTIPOSOLICITACAO, IDPRIORIDADEPADR, NRORG, DTINCLUSAO)
                    VALUES (:NRTIPOSOLICITACAO, :NMTIPOSOLICITACAO, :IDPRIORIDADEPADR, :NRORG, SYSDATE) ";

        $std = $this->connection->prepare($sql);

        $std->execute($typeParams);
    }
    
    // Atualiza a tabela de solicitação
    public function updateRequest($requestParams) {
        $sql = "UPDATE FCM_SOLICITACAO
                    SET NMSOLICITACAO = :NMSOLICITACAO,
                        DTSOLICITACAO = SYSDATE,
                        IDSITUACAO = :IDSITUACAO,
                        NRTIPOSOLICITACAO = :NRTIPOSOLICITACAO,
                        IDPRIORIDADE = :IDPRIORIDADE,
                        NRFUNCRESPONSAVEL = :NRFUNCRESPONSAVEL,
                        DSSOLICITACAO = :DSSOLICITACAO,
                        DTULTATU = SYSDATE
                 WHERE NRSOLICITACAO = :NRSOLICITACAO";

        $std = $this->connection->prepare($sql);

        $std->execute($requestParams);
        
    }
    
    // Cria uma tarefa
    public function createTask($checklistParams){
        $sql="INSERT INTO FCM_CHECKLISTAGEND (NRCHECKLISTAGEND, NRLOCALCLIENTE, NRCHECKLIST,                                                                             NRFUNCIONARIO, DTHORAAGEND, NRORG, DTINCLUSAO, 
                                                NMCHECKLISTAGEND, DTHRFINALAGEND)
                    VALUES (:NRCHECKLISTAGEND, :NRLOCALCLIENTE, :NRCHECKLIST, :NRFUNCIONARIO,
                            SYSDATE, :NRORG, SYSDATE, :NMCHECKLISTAGEND, SYSDATE+0.01)"; // sysdate + 15 min 
        $std = $this->connection->prepare($sql);
        $std->execute($checklistParams);
    }
    
    // Cria uma relação da tarefa com solicitação
    public function createTaskRequest($taskRequestParams){
        $sql="INSERT INTO FCM_SOLICITACAOTAREFAS (NRSOLICITACAOTAREFAS, NRSOLICITACAO, NRCHECKLISTAGEND,
                                                    NRORG, DTINCLUSAO)
                    VALUES (:NRSOLICITACAOTAREFAS, :NRSOLICITACAO, :NRCHECKLISTAGEND, :NRORG,
                            SYSDATE)";
        $std = $this->connection->prepare($sql);
        $std->execute($taskRequestParams);
    }
    
        // insere um dado no histórico
    public function insertHistoricData($historicParams){
        $sql="INSERT INTO FCM_SOLICITACAOHISTORICO (NRSOLICITACAOHISTORICO, NRSOLICITACAO, NMSOLICITACAO, DTHISTSOLICITACAO,                                             IDSITUACAO, IDTIPOHISTORICO, NRFUNCSOLICITANTE, DSSOLICITACAO, NRORG,
                                                    DTINCLUSAO)
                    VALUES (:NRSOLICITACAOHISTORICO, :NRSOLICITACAO, :NMSOLICITACAO, :DTHISTSOLICITACAO,
                            :IDSITUACAO, :IDTIPOHISTORICO, :NRFUNCSOLICITANTE, :DSSOLICITACAO, :NRORG, SYSDATE)";
        $std = $this->connection->prepare($sql);
        $std->execute($historicParams);
    }
    
    // Atualiza a tabela de solicitação
    public function updateRequestType($typeParams) {
        $sql = "UPDATE FCM_TIPOSOLICITACAO
                    SET NMTIPOSOLICITACAO = :NMTIPOSOLICITACAO,
                        IDPRIORIDADEPADR = :IDPRIORIDADEPADR,
                        DTULTATU = SYSDATE
                 WHERE NRTIPOSOLICITACAO = :NRTIPOSOLICITACAO";

        $std = $this->connection->prepare($sql);

        $std->execute($typeParams);
        
    }
    
    public function getLocalRequest($params) {
        $sqlCode = "SELECT FCM_LOCALCLI.NRLOCALCLIENTE, FCM_LOCALCLI.NMLOCALCLIENTE,
                           FCM_LOCALCLI.NRSETOR, FCM_LOCALCLI.NRLOCALCLIENTEPAI
                     FROM FCM_LOCALCLI
                     JOIN FCM_TIPOLOCALCLI
                       ON FCM_LOCALCLI.NRTIPOLOCALCLI = FCM_TIPOLOCALCLI.NRTIPOLOCALCLI
                    WHERE FCM_TIPOLOCALCLI.IDTIPOLOCALCLI = ".$params["parent"];

        $std = $this->connection->prepare($sqlCode);
        $std->execute();
        return $std->fetchAll();
    }
    
    public function insertNewRequest($params) {

      $sql="INSERT INTO FCM_SOLICITACAO (NRSOLICITACAO, NMSOLICITACAO, DTSOLICITACAO, IDSITUACAO, NRLOCALCLIENTE,
            NRFUNCABERTURA, DSSOLICITACAO, NRORG, IDATIVO, DTINCLUSAO)
              VALUES (:NRSOLICITACAO, :NMSOLICITACAO, SYSDATE, :IDSITUACAO, :NRLOCALCLIENTE, :NRFUNCABERTURA,
                      :DSSOLICITACAO, :NRORG, 'S', SYSDATE)";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }

    public function insertRequestHistory($params) {

      $sql="INSERT INTO FCM_SOLICITACAOHISTORICO (NRSOLICITACAOHISTORICO, NRSOLICITACAO, DTHISTSOLICITACAO,
            IDSITUACAO, IDTIPOHISTORICO, NRFUNCSOLICITANTE, DSSOLICITACAO, IDSATISFACAOSOLICITA, NRORG, IDATIVO)
              VALUES (:NRSOLICITACAOHISTORICO, :NRSOLICITACAO, SYSDATE, :IDSITUACAO, :IDTIPOHISTORICO,
              :NRFUNCSOLICITANTE, :DSSOLICITACAO, :IDSATISFACAOSOLICITA, :NRORG, 'S')";

        $std = $this->connection->prepare($sql);
        $std->execute($params);

    }

    public function getSituationRequest() {

        $sql = "SELECT DISTINCT S.IDSITUACAO AS NRIDSITUACAO,
                  CASE S.IDSITUACAO WHEN '1' THEN 'Pendente'
                     WHEN '2' THEN 'Em andamento'
                     WHEN '3' THEN 'Aguardando Avaliação'
                     WHEN '4' THEN 'Concluído'
                     ELSE NULL END AS IDSITUACAO
                     FROM FCM_SOLICITACAO S";

        $std = $this->connection->prepare($sql);
        $std->execute();
        return $std->fetchAll();
    }

    public function getNameRequest() {

        $sql = "SELECT S.NMSOLICITACAO, S.NRSOLICITACAO
                  FROM FCM_SOLICITACAO S";

        $std = $this->connection->prepare($sql);
        $std->execute();
        return $std->fetchAll();
    }

    public function getCardsRequests($parametros, $usersIds) {

        $sql = "SELECT S.NRSOLICITACAO, S.NMSOLICITACAO, TO_CHAR(S.DTSOLICITACAO, 'DD/MM HH24:MI') DTSOLICITACAO, S.NRLOCALCLIENTE, S.IDSITUACAO, LC.NMLOCALCLIENTE,
                  CASE S.IDSITUACAO
                    WHEN '1' THEN 'Pendente'
                    WHEN '2' THEN  'Em andamento'
                    WHEN '3' THEN 'Aguardando Avaliação'
                    WHEN '4' THEN 'Concluído'
                    ELSE NULL END AS IDSITUACAO
                 FROM FCM_SOLICITACAO S
                 JOIN FCM_LOCALCLI LC
                   ON LC.NRLOCALCLIENTE = S.NRLOCALCLIENTE
                   WHERE S.NRFUNCABERTURA IN ({$usersIds})"."$parametros ORDER BY DTSOLICITACAO DESC";

        $std = $this->connection->prepare($sql);
        $std->execute();
        return $std->fetchAll();

    }

    public function showRequest($param) {

        $sql = "SELECT S.NRSOLICITACAO, S.NMSOLICITACAO, S.DSSOLICITACAO, TO_CHAR(S.DTSOLICITACAO, 'DD/MM HH24:MI') DTSOLICITACAO, S.NRLOCALCLIENTE, S.IDSITUACAO, LC.NMLOCALCLIENTE,
                    CASE S.IDSITUACAO
                    WHEN '1' THEN 'Pendente'
                    WHEN '2' THEN  'Em andamento'
                    WHEN '3' THEN 'Aguardando Avaliação'
                    WHEN '4' THEN 'Concluído'
                    ELSE NULL END AS IDSITUACAO
                FROM FCM_SOLICITACAO S
                JOIN FCM_LOCALCLI LC
                ON LC.NRLOCALCLIENTE = S.NRLOCALCLIENTE
                WHERE S.NRSOLICITACAO = :NRSOLICITACAO";

        $std = $this->connection->prepare($sql);
        $std->execute($param);
        return $std->fetchAll();

    }

    public function upRequest($parametros) {

        $sql = "UPDATE FCM_SOLICITACAO
                  SET IDSITUACAO = :IDSITUACAO,
                      DSSOLICITACAO = :DSSOLICITACAO
                WHERE NRSOLICITACAO = :NRSOLICITACAO";

        $std = $this->connection->prepare($sql);
        $std->execute($parametros);
    }
    
    public function insertImg($param){
        $sql="INSERT INTO FCM_IMGSOLICITACAO (NRIMGSOLICITACAO, NRSOLICITACAO, DSCAMINHO, NRORG, IDATIVO, DSTYPE, NRSIZE, CDOPERINCLUSAO, DTINCLUSAO)
              VALUES (:NRIMGSOLICITACAO, :NRSOLICITACAO, :DSCAMINHO, :NRORG, 'S', :DSTYPE, :NRSIZE, :CDOPERINCLUSAO, SYSDATE)";
        $std = $this->connection->prepare($sql);
        $std->execute($param);
    }
    
}
?>